unit rkTreeView;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, Controls, ComCtrls, ExtCtrls;

type

  TrkTreeEditMode = (
    temNone,
    temKey,
    temMouse,
    temBoth
  );

  { TrkCustomTreeView }

  TrkCustomTreeView = class(TCustomTreeview)
  private
    FEditTimer: TTimer;
    FEditNode: TTreeNode;
    FEditInterval: Cardinal;
    FEditMode: TrkTreeEditMode;
    function GetReadOnly: Boolean;
    procedure SetEditInterval(AValue: Cardinal);
    procedure SetEditMode(AValue: TrkTreeEditMode);
  protected
    procedure Click; override;
    procedure DblClick; override;
    procedure DoExit; override;
    procedure EditTimerTimer(Sender: TObject); virtual;
    procedure SetOptions(NewOptions: TTreeViewOptions); override;
    procedure KeyDown(var Key : Word; Shift : TShiftState); override;
    property ReadOnly: Boolean read GetReadOnly;
    property EditInterval: Cardinal read FEditInterval write SetEditInterval;
    property EditMode: TrkTreeEditMode read FEditMode write SetEditMode default temNone;
  public
    constructor Create(AnOwner: TComponent); override;
  end;

const
  rkTreeViewOptions = [tvoAutoItemHeight, tvoKeepCollapsedNodes,
                       tvoReadOnly, tvoShowRoot, tvoShowLines,
                       tvoShowButtons, tvoToolTips, tvoThemedDraw];

type

  { TrkTreeView }

  TrkTreeView = class(TrkCustomTreeview)
  public
    property EditInterval;
    property ReadOnly;
  published
    property Align;
    property Anchors;
    property AutoExpand;
    property BorderSpacing;
    property BackgroundColor;
    property BorderStyle;
    property BorderWidth;
    property Color;
    property Constraints;
    property DefaultItemHeight;
    property DragKind;
    property DragCursor;
    property DragMode;
    property EditMode;
    property Enabled;
    property ExpandSignColor;
    property ExpandSignSize;
    property ExpandSignType;
    property Font;
    property HideSelection;
    property HotTrack;
    property HotTrackColor;
    property Images;
    property ImagesWidth;
    property Indent;
    property Items;
    property Options default rkTreeViewOptions;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property RightClickSelect;
    property RowSelect;
    property ScrollBars;
    property SelectionColor;
    property SelectionFontColor;
    property SelectionFontColorUsed;
    property SeparatorColor;
    property ShowButtons;
    property ShowHint;
    property ShowLines;
    property ShowRoot;
    property SortType;
    property StateImages;
    property StateImagesWidth;
    property TabOrder;
    property TabStop default True;
    property Tag;
    property ToolTips;
    property TreeLineColor;
    property TreeLinePenStyle;
    property Visible;
    property OnAddition;
    property OnAdvancedCustomDraw;
    property OnAdvancedCustomDrawItem;
    property OnChange;
    property OnChanging;
    property OnClick;
    property OnCollapsed;
    property OnCollapsing;
    property OnCompare;
    property OnContextPopup;
    property OnCreateNodeClass;
    property OnCustomCreateItem;
    property OnCustomDraw;
    property OnCustomDrawItem;
    property OnCustomDrawArrow;
    property OnDblClick;
    property OnDeletion;
    property OnDragDrop;
    property OnDragOver;
    property OnEdited;
    property OnEditing;
    property OnEditingEnd;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnExpanded;
    property OnExpanding;
    property OnGetImageIndex;
    property OnGetSelectedIndex;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnNodeChanged;
    property OnResize;
    property OnSelectionChanged;
    property OnShowHint;
    property OnStartDrag;
    property OnUTF8KeyPress;
  end;


implementation

uses LCLType, LCLIntf;

{ TrkCustomTreeView }

constructor TrkCustomTreeView.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  Options := rkTreeViewOptions;
  FEditInterval := GetDoubleClickTime + 150;
end;

function TrkCustomTreeView.GetReadOnly: Boolean;
begin
  Result := (FEditMode = temNone);
end;

procedure TrkCustomTreeView.SetOptions(NewOptions: TTreeViewOptions);
begin
  // force tvoReadOnly to prevent standard editing
  Include(NewOptions,tvoReadOnly);
  inherited SetOptions(NewOptions);
end;

procedure TrkCustomTreeView.DoExit;
begin
  FEditNode := nil;
  inherited DoExit;
end;

procedure TrkCustomTreeView.SetEditInterval(AValue: Cardinal);
begin
  if (FEditInterval <> AValue) then
  begin
    if Assigned(FEditTimer) then
      FEditTimer.Interval := AValue;
    FEditInterval := AValue;
  end;
end;

procedure TrkCustomTreeView.SetEditMode(AValue: TrkTreeEditMode);
begin
  if (FEditMode <> AValue) then
  begin
    if (AValue in [temMouse,temBoth]) then
    begin
      if not Assigned(FEditTimer) then
      begin
        FEditTimer := TTimer.Create(Self);
        FEditTimer.Interval := FEditInterval;
        FEditTimer.OnTimer := @EditTimerTimer;
      end;
    end else
      if Assigned(FEditTimer) then
        FreeAndNil(FEditTimer);
    FEditMode := AValue;
  end;
end;

procedure TrkCustomTreeView.Click;
begin
  if Assigned(FEditTimer) then
  begin
    if Assigned(Selected)
    and (FEditNode = Selected) then
      FEditTimer.Enabled := True
    else
      FEditTimer.Enabled := False;
    FEditNode := Selected;
  end;
  inherited Click;
end;

procedure TrkCustomTreeView.DblClick;
begin
  if Assigned(FEditTimer) then
    FEditTimer.Enabled := False;
  inherited DblClick;
end;

procedure TrkCustomTreeView.EditTimerTimer(Sender: TObject);
begin
  FEditTimer.Enabled := False;
  if Assigned(Selected) then Selected.EditText;
end;

procedure TrkCustomTreeView.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_F2)
  and (Shift = [])
  and Assigned(Selected)
  and (FEditMode in [temKey,temBoth]) then
    Selected.EditText
  else
    inherited KeyDown(Key, Shift);
end;

end.

